package java.util;

import java.io.InvalidObjectException;
import sun.misc.SharedSecrets;
import java.io.ObjectInputStream;
import java.io.IOException;
import java.io.ObjectOutputStream;
import java.io.Serializable;

public class HashSet<E> extends AbstractSet<E> implements Set<E>, Cloneable, Serializable
{
    static final long serialVersionUID = -5024744406713321676L;
    private transient HashMap<E, Object> map;
    private static final Object PRESENT;
    
    public HashSet() {
        this.map = new HashMap();
    }
    
    public HashSet(Collection<? extends E> collection) {
        this.map = new HashMap(Math.max((int)((float)collection.size() / 0.75f) + 1, 16));
        this.addAll(collection);
    }
    
    public HashSet(int n, float n2) {
        this.map = new HashMap(n, n2);
    }
    
    public HashSet(int n) {
        this.map = new HashMap(n);
    }
    
    HashSet(int n, float n2, boolean b) {
        this.map = new LinkedHashMap(n, n2);
    }
    
    @Override
    public Iterator<E> iterator() {
        return this.map.keySet().iterator();
    }
    
    @Override
    public int size() {
        return this.map.size();
    }
    
    @Override
    public boolean isEmpty() {
        return this.map.isEmpty();
    }
    
    @Override
    public boolean contains(Object o) {
        return this.map.containsKey(o);
    }
    
    @Override
    public boolean add(E e) {
        return this.map.put(e, HashSet.PRESENT) == null;
    }
    
    @Override
    public boolean remove(Object o) {
        return this.map.remove(o) == HashSet.PRESENT;
    }
    
    @Override
    public void clear() {
        this.map.clear();
    }
    
    public Object clone() {
        try {
            HashSet<Object> set = (HashSet)super.clone();
            set.map = (HashMap)this.map.clone();
            return set;
        }
        catch (CloneNotSupportedException cause) {
            throw new InternalError(cause);
        }
    }
    
    private void writeObject(ObjectOutputStream objectOutputStream) throws IOException {
        objectOutputStream.defaultWriteObject();
        objectOutputStream.writeInt(this.map.capacity());
        objectOutputStream.writeFloat(this.map.loadFactor());
        objectOutputStream.writeInt(this.map.size());
        Iterator<E> iterator = this.map.keySet().iterator();
        while (iterator.hasNext()) {
            objectOutputStream.writeObject(iterator.next());
        }
    }
    
    private void readObject(ObjectInputStream objectInputStream) throws IOException, ClassNotFoundException {
        objectInputStream.defaultReadObject();
        int int1 = objectInputStream.readInt();
        if (int1 < 0) {
            throw new InvalidObjectException("Illegal capacity: " + int1);
        }
        float float1 = objectInputStream.readFloat();
        if (float1 <= 0.0f || Float.isNaN(float1)) {
            throw new InvalidObjectException("Illegal load factor: " + float1);
        }
        int int2 = objectInputStream.readInt();
        if (int2 < 0) {
            throw new InvalidObjectException("Illegal size: " + int2);
        }
        int n = (int)Math.min((float)int2 * Math.min(1.0f / float1, 4.0f), 1.0737418E9f);
        SharedSecrets.getJavaOISAccess().checkArray(objectInputStream, Map.Entry[].class, HashMap.tableSizeFor(n));
        this.map = ((this instanceof LinkedHashSet) ? new LinkedHashMap(n, float1) : new HashMap(n, float1));
        for (int i = 0; i < int2; ++i) {
            this.map.put(objectInputStream.readObject(), HashSet.PRESENT);
        }
        return;
    }
    
    @Override
    public Spliterator<E> spliterator() {
        return new HashMap.KeySpliterator(this.map, 0, -1, 0, 0);
    }
    
    static {
        PRESENT = new Object();
    }
}
